/*
 *
 * Copyright (c) 1996 - 1998
 * Frank Buss (fb@zuse.com), Stephan Schloepke (stephan@nordrhein.net)
 *
 * This applet was published in the book:
 *
 *     "Programmier Training Java, In 15 Stunden topfit"
 *     ISBN 3-8158-1303-4
 *     Authors: Frank Buss, Stephan Schloepke
 *     Data Becker Verlage (http://www.databecker.de)
 *
 * You can get this applet and more at http://www.zuse.com
 *
 */

//
// Das Spielfeld fr TicTacToe, mit Eingabeauswertung
//

import java.awt.*;

public class Spielfeld extends Canvas {

    // Referenzen auf Objekte
    private TicTacToe dasApplet;  // Referenz auf das Applet
    private Image Kreuz;  // die Kreuz-Grafik
    private Image Kreis;  // die Kreis-Grafik
    private int Feld[][];  // das Spielfeld: 0=leer, 1=Kreis, 2=Kreuz

    // Variablen einfacher Datentypen
    private static int Breite=44;  // Breite und Hhe eines Kstchens
    private boolean Spieler;  // aktueller Spieler: true=Kreis, false=Kreuz
    private int ZugNr;  // Anzahl Zge im Spiel; bei 9 unentschieden

    // Konstruktor zum setzen der Klassenvariablen, zum erzeugen des Spielfeld-Arrays
    // und zum festlegen der Gre des Canvas
    public Spielfeld(TicTacToe dasApplet,Image Kreuz,Image Kreis) {
        this.dasApplet=dasApplet;
        this.Kreuz=Kreuz;
        this.Kreis=Kreis;
        Feld=new int[3][3];
        resize(3*Breite+4,3*Breite+4);
    }     

    // neues Spiel initialisieren:
    // Spielfeld leeren, Anzahl Zge auf 0 setzen und Spieler Kreuz auswhlen
    public void init() {
        for (int x=0;x<3;x++) {
            for (int y=0;y<3;y++) {
                Feld[x][y]=0;
            }
        }
        ZugNr=0;
        Spieler=false;
    }

    // testet, ob in der Spalte y des Spielfeld nur gleiche Symbole sind
    private boolean testHorz(int y) {
        return Feld[0][y]==Feld[1][y]&&Feld[1][y]==Feld[2][y];
    }

    // testet, ob in der Zeile x des Spielfeld nur gleiche Symbole sind
    private boolean testVert(int x) {
        return Feld[x][0]==Feld[x][1]&&Feld[x][1]==Feld[x][2];
    }

    // tested, ob von links oben nach rechts unten nur gleiche Symbole sind
    private boolean testLoRu() {
        return Feld[0][0]==Feld[1][1]&&Feld[1][1]==Feld[2][2];
    }

    // tested, ob von rechts oben nach links unten nur gleiche Symbole sind
    private boolean testRoLu() {
        return Feld[2][0]==Feld[1][1]&&Feld[1][1]==Feld[0][2];
    }

    // testet, ob in einer der beiden Schrgen nur gleiche Symbole sind
    private boolean testSchr(int x,int y) {
        switch(x+y*3) {

            // an den vier Kanten kann es keine Schrge mit drei gleichen Symbolen geben
            case 1:
            case 3:
            case 5:
            case 7:
                return false;

            // fr links oben oder rechts unten von links oben nach rechts unten testen
            case 0:
            case 8:
                return testLoRu();

            // fr rechts oben oder links unten von rechts oben nach links unten testen
            case 2:
            case 6:
                return testRoLu();

            // fr die Mitte beide Schrgen testen
            case 4:
                return testLoRu()||testRoLu();
        }
        return false;
    }

    // tested, ob waagerecht, senkrecht oder schrg drei gleiche Symbole sind
    private boolean Test(int x,int y) {
        return testHorz(y)||testVert(x)||testSchr(x,y);
    }

    // wird bei Druck auf die Maustaste aufgerufen und setzt ggf. ein Symbol ins Spielfeld
    public boolean mouseDown(Event evt,int x,int y) {
        // errechnen der logischen Feldarray-Koordinate
        int xp=x/Breite;
        int yp=y/Breite;

        // nur auswerten, wenn innerhalb des Spielfelds
        if (xp>=0&&xp<3&&yp>=0&&yp<3) {

            // nur ein Symbol setzen, wenn noch frei ist
            if (Feld[xp][yp]==0) {

                // Kreis oder Kreuz setzen, Spieler wechseln und Zugnummer erhhen
                if (Spieler) Feld[xp][yp]=1; else Feld[xp][yp]=2;
                Spieler=!Spieler;
                ZugNr++;

                // falls drei voll sind, dann hat der Spieler gewonnen, auch wenn das Feld
                // voll ist. Ansonsten unentschieden (0), wenn das Feld voll ist
                if (Test(xp,yp)) dasApplet.Gewonnen(Feld[xp][yp]); else {
                    if (ZugNr==9) dasApplet.Gewonnen(0);
                }

                // neuen Zug anzeigen
                repaint();
            }
        }

        // Status true zurckliefern, als Anzeige, da der Mausklick ausgewertet wurde
        return true;
    }

    // Anzeige des Spielfelds
    public void paint(Graphics g) {

        // Schatten malen
        g.setColor(Color.black);
        g.fillRect(4,4,3*Breite,3*Breite);

        // weien Untergrund erzeugen
        g.setColor(Color.white);
        g.fillRect(0,0,3*Breite,3*Breite);

        // Linien des Spielfeldes zeichnen
        g.setColor(Color.black);
        for (int c=0;c<=3;c++) {

            // senkrechte Linien ziehen
            g.drawRect(c*Breite,0,1,3*Breite+1);

            // waagerechte Linien ziehen
            g.drawRect(0,c*Breite,3*Breite+1,1);
        }

        // Kreuze und Kreise einzeichnen
        for (int x=0;x<3;x++) {
            for (int y=0;y<3;y++) {
                switch (Feld[x][y]) {
                    case 1:  // Kreis zeichnen
                        g.drawImage(Kreis,x*Breite+2,y*Breite+2,dasApplet);
                        break;
                    case 2:  // Kreuz an die Stelle setzen
                        g.drawImage(Kreuz,x*Breite+2,y*Breite+2,dasApplet);
                }  // ende switch
            }  // ende for y
        }  // ende for x
    }  // ende paint-Methode

}  // ende Applet
